=begin
#Vericred API

## Getting Started

## Signing Up

Visit our [Developer Portal](https://developers.vericred.com) to
create an account.

Once you have created an account, you can create one Application for your
Production environment and another for a Sandbox (select the appropriate Plan when you create the Application).

## SDKs

Our API follows standard REST conventions, so you can use any HTTP client
to integrate with us. You will likely find it easier to use one of our
[autogenerated SDKs](https://github.com/vericred/?query=vericred-),
which we make available for several common programming languages.

## Authentication

To authenticate, pass the API Key you created in the Developer Portal as
a `Vericred-Api-Key` header.

`curl -H 'Vericred-Api-Key: YOUR_KEY' "https://api.vericred.com/drugs?search_term=Lipitor"`

## Versioning

Vericred's API default to the latest version.  However, if you need a specific
version, you can request it with an `Accept-Version` header.

The current version is `v6`.  We support prior versions. Our latest
stable version is `v7` and we will indicate in the documentation for an
endpoint if it's required.

`curl -H 'Vericred-Api-Key: YOUR_KEY' -H 'Accept-Version: v6' "https://api.vericred.com/drugs?search_term=Lipitor"`

## Pagination

Endpoints that accept `page` and `per_page` parameters are paginated. They expose
four additional fields that contain data about your position in the response,
namely `Total`, `Per-Page`, `Link`, and `Page` as described in [RFC-5988](https://tools.ietf.org/html/rfc5988).

For example, to display 5 results per page and view the second page of a
`GET` to `/networks`, your final request would be `GET /networks?....page=2&per_page=5`.

## Sideloading

When we return multiple levels of an object graph (e.g. `Provider`s and their
`State`s we typically the associated data.  In this example, we would
provide an Array of `State`s and a `state_id` for each provider.  This is
done primarily to reduce the payload size since many of the `Provider`s
will share a `State`

### Simplified Example
```
{
  providers: [{ id: 1, state_id: 1}, { id: 2, state_id: 1 }],
  states: [{ id: 1, code: 'NY' }]
}
```

If you need the second level of the object graph, you can just match the
corresponding id.

## Selecting specific data

All endpoints allow you to specify which fields you would like to return.
This allows you to limit the response to contain only the data you need.

For example, let's take a request that returns the following JSON by default

```
{
  provider: {
    id: 1,
    name: 'John',
    phone: '1234567890',
    field_we_dont_care_about: 'value_we_dont_care_about'
  },
  states: [{
    id: 1,
    name: 'New York',
    code: 'NY',
    field_we_dont_care_about: 'value_we_dont_care_about'
  }]
}
```

To limit our results to only return the fields we care about, we specify the
`select` in the query string for a `GET` or the body for a `POST`.

In this case, we want to select `name` and `phone` from the `provider` key,
so we would add the parameters `select=provider.name,provider.phone`.
We also want the `name` and `code` from the `states` key, so we would
add the parameters `select=states.name,states.code`.  The id field of
each document is always returned whether or not it is requested.

Our final request would be `GET /providers/1234567898?select=provider.name,provider.phone,states.name,states.code`

The response would be

```
{
  provider: {
    id: 1234567898,
    name: 'John',
    phone: '1234567890'
  },
  states: [{
    id: 1,
    name: 'New York',
    code: 'NY'
  }]
}
```

# Plan and Rate Data

Vericred's Plan and Rate Data let you search and quote Major Medical and Ancillary Insurance Plans in a given area for a particular family in the Individual Market or a group of families in the Small Group Market.  Vericred provides the relevant data via this API and via our Bulk Format (documented [below](#Bulk Plan and Rate Data))

## Plans

A Plan defines a set of Benefits available to its purchaser.  For example, a Major Medical Plan would specify cost-share Benefits for services like a Primary Care Provider visit, a Specialist visit or an Emergency Room visit.  A Dental Plan might specify Benefits for Periodontics and Fluroride Treatments.  The Benefits for each Product type ([Major Medical](#header-major-medical), [Dental](#header-dental), and [Vision](#header-vision)) are documented below.

### Benefits Format

Benefits for Plans can be quite complex.  With the goals of capturing and standardizing the complexity while retaining a human-readable format, we have developed a [Bakus-Naur Form](https://en.wikipedia.org/wiki/Backus%E2%80%93Naur_form)(BNF) context-free grammar, with which we present Benefits.

Benefit cost-share strings are formatted to capture:
 * Network tiers
 * Compound or conditional cost-share
 * Limits on the cost-share
 * Benefit-specific maximum out-of-pocket costs

**Example #1**
As an example, we would represent [this Summary of Benefits &amp; Coverage](https://s3.amazonaws.com/vericred-data/SBC/2017/33602TX0780032.pdf) as:

* **Hospital stay facility fees**:
  - Network Provider: `$400 copay/admit plus 20% coinsurance`
  - Out-of-Network Provider: `$1,500 copay/admit plus 50% coinsurance`
  - Vericred's format for this benefit: `In-Network: $400 before deductible then 20% after deductible / Out-of-Network: $1,500 before deductible then 50% after deductible`

* **Rehabilitation services:**
  - Network Provider: `20% coinsurance`
  - Out-of-Network Provider: `50% coinsurance`
  - Limitations & Exceptions: `35 visit maximum per benefit period combined with Chiropractic care.`
  - Vericred's format for this benefit: `In-Network: 20% after deductible / Out-of-Network: 50% after deductible | limit: 35 visit(s) per Benefit Period`

**Example #2**
In [this other Summary of Benefits &amp; Coverage](https://s3.amazonaws.com/vericred-data/SBC/2017/40733CA0110568.pdf), the **specialty_drugs** cost-share has a maximum out-of-pocket for in-network pharmacies.
* **Specialty drugs:**
  - Network Provider: `40% coinsurance up to a $500 maximum for up to a 30 day supply`
  - Out-of-Network Provider `Not covered`
  - Vericred's format for this benefit: `In-Network: 40% after deductible, up to $500 per script / Out-of-Network: 100%`

**BNF**

Here's a description of the benefits summary string, represented as a context-free grammar:

```
root                      ::= coverage

coverage                  ::= (simple_coverage | tiered_coverage) (space pipe space coverage_modifier)?
tiered_coverage           ::= tier (space slash space tier)*
tier                      ::= tier_name colon space (tier_coverage | not_applicable)
tier_coverage             ::= simple_coverage (space (then | or | and) space simple_coverage)* tier_limitation?
simple_coverage           ::= (pre_coverage_limitation space)? coverage_amount (space post_coverage_limitation)? (comma? space coverage_condition)?
coverage_modifier         ::= limit_condition colon space (((simple_coverage | simple_limitation) (semicolon space see_carrier_documentation)?) | see_carrier_documentation | waived_if_admitted | shared_across_tiers)
waived_if_admitted        ::= ("copay" space)? "waived if admitted"
simple_limitation         ::= pre_coverage_limitation space "copay applies"
tier_name                 ::= "In-Network-Tier-2" | "Out-of-Network" | "In-Network"
limit_condition           ::= "limit" | "condition"
tier_limitation           ::= comma space "up to" space (currency | (integer space time_unit plural?)) (space post_coverage_limitation)?
coverage_amount           ::= currency | unlimited | included | unknown | percentage | (digits space (treatment_unit | time_unit) plural?)
pre_coverage_limitation   ::= first space digits space time_unit plural?
post_coverage_limitation  ::= (((then space currency) | "per condition") space)? "per" space (treatment_unit | (integer space time_unit) | time_unit) plural?
coverage_condition        ::= ("before deductible" | "after deductible" | "penalty" | allowance | "in-state" | "out-of-state") (space allowance)?
allowance                 ::= upto_allowance | after_allowance
upto_allowance            ::= "up to" space (currency space)? "allowance"
after_allowance           ::= "after" space (currency space)? "allowance"
see_carrier_documentation ::= "see carrier documentation for more information"
shared_across_tiers       ::= "shared across all tiers"
unknown                   ::= "unknown"
unlimited                 ::= /[uU]nlimited/
included                  ::= /[iI]ncluded in [mM]edical/
time_unit                 ::= /[hH]our/ | (((/[cC]alendar/ | /[cC]ontract/) space)? /[yY]ear/) | /[mM]onth/ | /[dD]ay/ | /[wW]eek/ | /[vV]isit/ | /[lL]ifetime/ | ((((/[bB]enefit/ plural?) | /[eE]ligibility/) space)? /[pP]eriod/)
treatment_unit            ::= /[pP]erson/ | /[gG]roup/ | /[cC]ondition/ | /[sS]cript/ | /[vV]isit/ | /[eE]xam/ | /[iI]tem/ | /[sS]tay/ | /[tT]reatment/ | /[aA]dmission/ | /[eE]pisode/
comma                     ::= ","
colon                     ::= ":"
semicolon                 ::= ";"
pipe                      ::= "|"
slash                     ::= "/"
plural                    ::= "(s)" | "s"
then                      ::= "then" | ("," space) | space
or                        ::= "or"
and                       ::= "and"
not_applicable            ::= "Not Applicable" | "N/A" | "NA"
first                     ::= "first"
currency                  ::= "$" number
percentage                ::= number "%"
number                    ::= float | integer
float                     ::= digits "." digits
integer                   ::= /[0-9]/+ (comma_int | under_int)*
comma_int                 ::= ("," /[0-9]/*3) !"_"
under_int                 ::= ("_" /[0-9]/*3) !","
digits                    ::= /[0-9]/+ ("_" /[0-9]/+)*
space                     ::= /[ \t]/+
```


### Major Medical

Vericred's data covers all Major Medical Plans available in the Individual and Small Groups (2-50 or 2-100) Markets in the US.  These Plans are governed by CMS and are ACA-compliant.  We do not include certain Plans that fall outside of the ACA, for example, Faith-Based Plans or Short-Term Medical Plans

We support the following Benefits Fields for Major Medical Plans.  These represent the vast majority of fields available on a [Summary of Benefits and Coverage](https://www.healthcare.gov/health-care-law-protections/summary-of-benefits-and-coverage/)

The following are the appropriate Benefit Fields for Major Medical:

- ambulance
- child\_eye\_exam
- child\_eyewear
- chiropractic\_services
- diagnostic\_test
- durable\_medical\_equipment
- emergency\_room
- family\_drug\_deductible
- family\_drug\_moop
- family\_medical\_deductible
- family\_medical\_moop
- generic\_drugs
- habilitation\_services
- home\_health\_care
- hospice\_service
- imaging\_center
- imaging\_physician
- individual\_drug\_deductible
- individual\_drug\_moop
- individual\_medical\_deductible
- individual\_medical\_moop
- inpatient\_birth
- inpatient\_birth\_physician
- inpatient\_facility
- inpatient\_mental\_health
- inpatient\_physician
- inpatient\_substance
- lab\_test
- non\_preferred\_brand\_drugs
- nonpreferred\_generic\_drug\_share
- nonpreferred\_specialty\_drug\_share
- outpatient\_ambulatory\_care\_center
- outpatient\_facility
- outpatient\_mental\_health
- outpatient\_physician
- outpatient\_substance
- plan\_coinsurance
- postnatal\_care
- preferred\_brand\_drugs
- prenatal\_care
- preventative\_care
- primary\_care\_physician
- rehabilitation\_services
- skilled\_nursing
- specialist
- specialty\_drugs
- telemedicine
- urgent\_care

### Dental

Dental benefits are less standardized than [Major Medical](#header-major-medical).  Because of this, we have captured benefits for the most commonly specified services and procedures.  If a Plan only specifies cost-share for "Major", "Minor", "Elective", etc, we determine the category for each of the benefits that we support and display the appropriate value for its category.

To view the technical documentation, [click here](#dental-plans).

The following are the supported Benefit Fields for Dental:

- bridges
- crowns
- denture\_relines\_rebases
- denture\_repair\_and\_adjustments
- dentures
- emergency\_treatment
- endodontics
- family\_annual\_max
- family\_deductible
- fluoride\_treatment
- implants
- individual\_annual\_max
- individual\_deductible
- inlays
- onlays
- oral\_exam
- oral\_surgery
- orthodontics\_adult
- orthodontics\_child
- periodontal\_maintenance
- periodontics
- prophylaxis\_cleaning
- radiograph\_bitewings
- radiograph\_other
- restoration\_fillings
- sealant
- simple\_extraction
- space\_maintainers

### Vision

Vision benefits are similar in structure to [Dental](#header-dental).  Again, when benefits are broken out by category, we determine the appropriate category for each service or procedure and display the approprate value for its category.

To view the technical documentation, [click here](#vision-plans).

The following are the supported Benefit Fields for Vision:

- eye\_exam
- retinal\_imaging
- frame
- eyeglass\_lenses\_single\_vision
- eyeglass\_lenses\_bifocal
- eyeglass\_lenses\_trifocal
- eyeglass\_lenses\_lenticular
- uv\_coating
- tint
- standard\_antireflective\_coating
- premium\_antireflective\_coating
- standard\_polycarbonate\_lenses\_child
- standard\_polycarbonate\_lenses\_adult
- standard\_progressive\_lenses
- premium\_progressive\_lenses
- standard\_scratch\_resistance
- polarized\_lenses
- photochromatic\_lenses
- standard\_contact\_lens\_fit\_and\_follow\_up
- premium\_contact\_lens\_fit\_and\_follow\_up
- contact\_lenses\_conventional
- contact\_lenses\_disposable
- contact\_lenses\_medically\_necessary
- laser\_vision\_correction
- additional\_pairs\_of\_eyeglasses

### Medicare Advantage

Medicare Advantage endpoints support MA (Medicare Advantage plans without prescription drug benefits), MAPD (Medicare Advantage plans with prescription drug benefits), and PDP (prescription drug benefits only) plans.

Medical and hospital benefits - we support over 4000 individual plans through our API nationwide. At this time, we do not support EGHP (employer group health plans).

Prescription drug benefits - drug benefits are categorized into initial coverage phase, coverage gap phase, and catastrophic phase. Each phase is broken down by drug tiers, pharmacy tiers, and length of supply.

[Click here](#header-medicare-advantage-quotes) to learn more about quoting Medicare Advantage plans.

## Rates

Rates are returned from the API as a part of [Quoting](#header-quoting).  We calculate Rates in one of two ways.

### Sheet Rates
When a Carrier supplies us with Sheet Rates, we display *exactly* the value provided to us.  For example, in the Major Medical market, most Carriers provide a single rate for each combination of Applicant age and tobacco status in a given [Rating Area](#header-rating-areas).  For example, in Austin, TX, a 21-year-old non-tobacco-user may be $312.41 per month while a 22-year-old tobacco-user may be $401.75 per month.  Certain Vision and Dental Carriers supply Sheet Rates as well, though it is less common.

### Rate Factors
Certain [Major Medical](#header-major-medical) Carriers and most [Vision](#header-vision) and [Dental](#header-dental) Carriers supply Rate Factors.  The attributes on which the factors are based are the same as [Sheet Rates](#header-sheet-rates) for the [Major Medical](#header-major-medical) market (due to restrictions on what factors may be used in ACA Plans, which limit the possible factors to age and tobacco status).

In [Dental](#header-dental) and [Vision](#header-vision), the types of [Rate Factors](#header-rate-factors) are more varied.  For example, SIC Code and Group size in the Group market and Gender in the Individual Market are commonly used [Rate Factors](#header-rate-factors)

Other common [Rate Factors](#header-rate-factors) for [Dental](#header-dental) and [Vision](#header-vision) products are Geographic and "Trend" (enrollment date) Factors.  In [Major Medical](#header-major-medical), these types of variance are handled by CMS-defined [Rating Areas](#header-rating-areas).

In order to calculate a Rate using [Rate Factors](#header-rate-factors), the following methodology is applied:

```
B = Base Rate
f = Rate Factor Function 1
f' = Rate Factor Function 2

B * f(x) * f'(y) [* f''(z)] ... = n
```

## Rating Areas
For [Major Medical](#header-major-medical) products, [CMS](https://www.cms.gov/) defines [Rating Areas](https://www.cms.gov/cciio/programs-and-initiatives/health-insurance-market-reforms/state-gra.html).  Under the ACA, *all* [Rate Factors](#header-rate-factors) in a Rating Area must be identical for a given time period.  E.g. in Arizona, the rate for a 21-year-old non-tobacco user must be identical in all counties contained in Rating Area 1 (Mohave, Coconino, Apache, and Navajo), but *may* be different than the rate for a 21-year-old non-tobacco user in all counties Rating Area 2 (Yavapai county only) for a given year in the Individual Market and a given quartern in the Small Group market.

[Rating Areas](#header-rating-areas) are defined either by County, Zip Code or both, depending on the State.  Because of this variance, all API endpoints that require a [Location](#header-specifying-the-location) require both `zip_code` and `fips_code` (a county code).  [Bulk Data](#bulkplanandratedata) for [Rating Areas](#bulkratingareas) and [Service Areas](#bulkserviceareas) also specifies locations using both `zip_code` and `fips_code`.

*Rating Areas do not apply to products other than [Major Medical](#header-major-medical)*

## Service Areas
CMS mandates that [Major Medical](#header-major-medical) Rates be defined by [Rating Areas](#header-rating-areas), which themselves define a geography in which Plans are offered.  Carriers often choose *not* to offer a Plan in and entire [Rating Area](#header-rating-areas) due to network coverage or other factors.  Instead, the Carrier would define a [Service Area](#header-service-areas) that specifies where a given Plan is offered.

Each Plan is available in a single [Service Area](#header-service-areas) and each [Service Area](#header-service-areas) is defined by either County, Zip Code, or both, depending on the Carrier.  Because of this variance, all API endpoints that require a [Location](#header-specifying-the-location) require both `zip_code` and `fips_code` (a county code). [Bulk Data](#bulkplanandratedata) for [Rating Areas](#bulkratingareas) and [Service Areas](#bulkserviceareas) also specifies locations using both `zip_code` and `fips_code`.

In [Dental](#header-dental) and [Vision](#header-vision) plans, we use a [Service Area](#header-service-areas) to define availability as well, although it typically mirrors a Geographic [Rate Factor](#header-rate-factors).

# Quoting

One of the primary use-cases for the Vericred API is to run Quotes to determine the Rate for a given family (in the Individual Market) or group (in the Small Group Market).  We support quoting across [Major Medical](#header-major-medical), [Vision](#header-vision), and [Dental](#header-dental).  In both cases, the process of generating a Quote is broken out into several steps:

1. Find all available [Plans](#header-plans) in the relevent [Service Areas](#header-service-areas) for the family or group.
1. Using [Business Rules](#header-business-rules) for each [Plan](#header-plans), determine if the family or group is eligible for that [Plan](#header-plans).
1. Using [Business Rules](#header-business-rules) for each [Plan](#header-plans), determine which members of the family or which members of each family in the group should be considered for Rating.
1. Using the [Sheet Rates](#header-sheet-rates) or [Rate Factors](#header-rate-factors) for each [Plan](#header-plans), determine the Rate the family, or for each family in the group.
1. If running a [Composite](#compositerates) quote, determine the portion of the total Rate that each family will pay.

## Individual Quotes
An Individual Quote is one for Plans that are available to a particular family, outside the context of their Employer.  In the [Major Medical](#header-major-medical) market, many of these Plans are available on [Healthcare.gov](https://www.healthcare.gov) or the State-Based Exchange for non-Healthcare.gov states.  The API supports both on-market and off-market Plans.

For details on Major Medical Quoting API calls see [below](#medicalplans-medicalplans-post)

### Specifying the Location
In order to determine which plans are [available](#header-service-areas) and the [rate](#header-rating-areas) for each Plan, you must specify a location.  When creating a [Quote](#header-quoting) for the Individual Market, that information is contained in the `POST` body of the request:
```
POST /plans/medical/search
{
  ...
  "zip_code": "11201",
  "fips_code": "36047"
  ...
}

```

### Specifying Applicants
Applicants are the members of the family being quoted and are specified in the `POST` body of the request.
```
POST /plans/medical/search
{
  ...
  "applicants": [
    {
      "age": 34,
      "smoker": true,
      "child": false
    },
    {
      "age": 32,
      "smoker": false,
      "child": false
    },
    {
      "age": 4,
      "smoker": false,
      "child": true
    }
  ]
  ...
}
```

### Specifying Enrollment Date
The `enrollment_date` determines which [Plans](#header-plans) and [Rates](#header-rates) are returned.  Specifying an `enrollment_date` in the past allows you to calculate historical data as far back as 2014.

### Plan Benefits
Plan Benefits are returned in the response for [Individual Quotes](#individualquotes)
```
POST /plans/medical/search
{
  ...
}

Response:
{
  "plans": [
    {
      ...
      "individual_medical_deductible": "$5,000",
      "family_medical_deductible": "$10,000"
      ...
    }

  ]
}

```

### Premiums
The value for the family being quoted is returned in the `premium` field.  If no [Applicants](#specifyingapplicants) are provided, the `premium` field will be `0`

### Major Medical Quotes
In order to [Quote](#header-quoting) [Major Medical](#header-major-medical) Plans, send a `POST` to `/plans/medical/search`.  In addition, the `age`, `smoker` and `child` attributes of each Applicant must be present.

#### Subsidies
On-market ([Healthcare.gov](https://www.healthcare.gov) and State-Based Exchange) [Major Medical](#header-major-medical) Plans are eligible for government subsidies.  The subsidy calculation is based on the percentage of the family's income that the IRS has designated as "affordable" for that family and the Second Lowest-Cost Silver Plan available to that family.

In order to calculate subsidies for a family the following parameters must be supplied:
```
POST /plans/medical/search
{
  ...
  "household_size": 4,
  "household_income": 40000
  ...
}
```

The amount that the family will pay after subsidy is returned in the `premium_subsidized` field for each plan.

##### Subsidy Calculation
Here is how subsidies are calculated.  This is fully handled by the Vericred API, but the steps are enumerated below for clarity.

1. Determine the percentage of the [Federal Poverty Level](https://aspe.hhs.gov/poverty-guidelines) for the family based on the household size and income.
1. Reference the [CMS table](https://www.irs.gov/pub/irs-drop/rp-17-36.pdf) to determine the appropriate percentage of income for the family to spend on healthcare.
1. Multiply that value by the family's income.  This is the total amount that the family can spend on healthcare for the year, after the subsidy.
1. Find the cost of the Second Least-Expensive Silver Plan available to the family, accounting for the percentage of premium that goes to [Essential Health Benefits](https://www.healthcare.gov/glossary/essential-health-benefits/)
1. Calculate the difference in price between the amount the family should spend on healthcare and the Second Least-Expensive Silver Plan's premium.  This is the subsidy.
1. Apply the subsidy to all on-market Plans available to the family.  The subsidized premium can never be below $0 (for example, a low-cost Bronze Plan may be less expensive than the subsidy)

#### Cost Sharing Reduction Plans
[Cost Sharing Reduction (CSR)](https://www.healthcare.gov/glossary/cost-sharing-reduction/) Plans are available to lower income families and offer enhanced benefits for certain Silver Plans at the same cost as the non-CSR Plans available to higher-income families.

If a family is eligible for CSR Plans, the Vericred API will return the relevant Plan in place of the non-CSR version.

In order to include CSR Plans where applicable, the following parameters must be supplied:
```
POST /plans/medical/search
{
  ...
  "household_size": 4,
  "household_income": 40000
  ...
}
```

#### The Children's Health Insurance Program (CHIP)
[The Children's Health Insurance Program (CHIP)](https://www.healthcare.gov/medicaid-chip/childrens-health-insurance-program/) provides low-cost health coverage to children in families that earn too much money to qualify for Medicaid.

If a family is eligible for CHIP, the Vericred API will include the subsidy in the returned premiums. In order to include CHIP subsidies in premium calculations, the following parameters must be supplied:
```
POST /plans/medical/search
{
  ...
  "applicants": [
    {
      "age": 34,
      "smoker": true,
      "child": false
    },
    {
      "age": 32,
      "smoker": false,
      "child": false
    },
    {
      "age": 4,
      "smoker": false,
      "child": true
    },
    {
      "age": 4,
      "smoker": false,
      "child": true
    }
  ],
  "household_size": 4,
  "household_income": 40000
  ...
}
```

CHIP eligibility is displayed within the meta-tag in the response as `eligible_for_chip_medicaid`:
```
{
  "meta": {
      "total": 150,
      "eligible_for_chip_medicaid": true,
      "premium_tax_credit": 500.20
    },
  "coverages": [],
  "plans": [...]
}
```

If you do not want CHIP subsidies to be included in the premiums, simply do not include the `household_income` parameter in the request.

### Dental Quotes
Quoting [Dental](#header-dental) Plans for a family requires slightly different parameters for [Applicants](#specifyingapplicants), due to the method with which Plans are [rated](#header-rate-factors).  The folloiwng example contains the require parameters:
```
POST /plans/dental/search
{
  ...
  "applicants": [
    {
      "age": 34,
      "gender": "M",
      "child": false
    },
    {
      "age": 32,
      "gender": "F",
      "child": false
    },
    {
      "age": 4,
      "gender": "M",
      "child": true
    }
  ]
  ...
}
```

Note that in contrast to [Major Medical Quotes](#majormedicalquotes), [Dental Quotes](#dentalquotes) require `gender`, but *do not* require `smoker`.

Also note that [Subsidies](#subsidies) and [Cost Sharing Reduction](#costsharingreductionplans) are *not* relevant for [Dental Quotes](#dentalquotes).

### Vision Quotes

Quoting [Vision](#header-vision) Plans for a family requires slightly different parameters for [Applicants](#specifyingapplicants), due to the method with which Plans are [rated](#header-rate-factors).  The folloiwng example contains the require parameters:
```
POST /plans/vision/search
{
  ...
  "applicants": [
    {
      "age": 34,
      "gender": "M",
      "child": false
    },
    {
      "age": 32,
      "gender": "F",
      "child": false
    },
    {
      "age": 4,
      "gender": "M",
      "child": true
    }
  ]
  ...
}
```

Note that in contrast to [Major Medical Quotes](#majormedicalquotes), [Vision Quotes](#visionquotes) require `gender`, but *do not* require `smoker`.

Also note that [Subsidies](#subsidies) and [Cost Sharing Reduction](#costsharingreductionplans) are *not* relevant for [Vision Quotes](#visionquotes).

### Medicare Advantage quotes

Given an enrollment date and location, you can search all plans available at this location using [/plans/medadv/search](#medicare-advantage-plans-medicare-advantage-plans-post); you can also search a specific plan using its plan ID with [/plans/medadv/{plan_id}](#medicare-advantage-plans-medicare-advantage-plans-get) if the plan ID is already known. The following example contains the required parameters:
```
{
"zip_code": "02880",
"fips_code": "44009",
"enrollment_date": "2019-01-01"
}
```
To view the technical documentation, [click here](#medicare-advantage-plans).


## Quotes for Groups
A Group Quote finds [Plans](#header-plans) and [Rates](#header-rates) for a group of employees for a small business.  Different [Plans](#header-plans) are available to small groups than are available in [Individual Quoting](#individualquotes).  In addition, [Business Rules](#header-business-rules) that apply across multiple families or based upon employer attributes such as [SIC](https://en.wikipedia.org/wiki/Standard_Industrial_Classification) code factor into rates and availability.

In addition, due to performance requirements and for enhanced auditing, [Group Quotes](#groupquotes) are persisted across requests.  This means that a given [Quote](#groupquotes) can be retrieved after it has been created.

### Identifiers
In order to make it easier to cross-reference local copies of data with [Quotes](#header-creating-a-quote) and other data in the Vericred API, most entities allow for the specification of an `external_id` field.  You can use this to store a primary or natural key from your system in order to easily match records returned from the API with records in your system.

### Specifying the Group
Creating a group is the first step in [Group Quoting](#groupquoting).  The API requires that certain information such as `sic_code`, and `chamber_association` be provided and returns a the attributes and `id` for the newly created `Group`

Full documentation is available [below](#groupscreate)

### Specifying the Locations
When creating a `Group`, you must also specify one or more `Location`s. Of those `Location`s specified, one must be `primary`.  That `Location` is used to calculate Plan eligibility using the relevant [Service Areas](#header-service-areas).  Some Carriers use secondary `Location`s to determine eligibility as well, which is why those must be specified as well.

```
POST /groups
{
  "group": {
    ...
  },
  "locations": [
    {
      ...
      "zip_code": "11201",
      "fips_code": "36047",
      "primary": true
      ...
    }
  ]
}
```

### Specifying the Census
A Census is the collection of `Member`s contained in the `Group`.  The attributes of each `Member` and his or her `Dependent`s determine the [Rate](#header-rates) for the `Group` as a whole.  Certain attributes of the `Member` are important for calculating [Rates](#header-rates) and applying [Business Rules](#header-business-rules).  For example, the `Member`'s home address and in which office he or she works are relevant for certain [Business Rules](#header-business-rules).

#### Dependent Relationships
The `Dependent`s for a given `Member` also factor into the [Rates](#header-rates) and application of [Business Rules](#header-business-rules).  For example, certain [Plans](#header-plans) cover only `Dependent`s of particular types and/or only `Dependent`s of a particular type who live in the same household as the primary `Member`

Valid Dependent Relationships:
- `adopted_child`
- `child`
- `court_appointed_guardian`
- `dependent_of_dependent`
- `ex_spouse`
- `foster_child`
- `grand_child`
- `guardian`
- `life_partner`
- `other`
- `sibling`
- `sponsored_dependent`
- `spouse`
- `step_child`
- `ward`

```
POST

/groups/{id}/members
{
  "members": [
    ...
    {
      "cobra": false,
      "date_of_birth": "1980-01-01",
      "fips_code": "36047"
      "gender": "M",
      "last_used_tobacco": "2017-01-01",
      "location_id": :location_id
      "retiree": false,
      "zip_code": "11201",
      "dependents": [
        ...
        {
          "relationship": "child",
          "same_household": true
        }
        ...
      ]
    }
    ...
  ]
}

```

### Creating a Quote
Once the [Census](#header-specifying-the-census) has been created, we can generate a `Quote` for the `Group`.

#### Major Medical Quotes
To generate a [Major Medical](#header-major-medical) Quote, specify the `product_line` of `Quote` as `medical`

```
POST /groups/{id}/quotes
{
  ...
  "product_line": "medical"
  ...
}
```

#### Dental Quotes
To generate a [Dental](#header-dental) Quote, specify the `product_line` of `Quote` as `dental`

```
POST /groups/{id}/quotes
{
  ...
  "product_line": "dental"
  ...
}
```

#### Vision Quotes
To generate a [Vision](#header-vision) Quote, specify the `product_line` of `Quote` as `vision`

```
POST /groups/{id}/quotes
{
  ...
  "product_line": "vision"
  ...
}
```

### Retrieving Aggregate Rates
Once you have created a [Quote](#header-creating-a-quote), you can retrieve its aggregate [Rates](#header-rates).  [Rates](#header-rates) are broken down by `Member` and `Dependent`, so that you can show the final cost in different scenarios where an employer might cover a different percentage of `Member` and `Dependent` costs.

```
GET /quotes/{id}/rates

Response
{
  "rates": [
    ...
    {
      "id": "123abc",
      "plan_id": "12345NY6789012",
      "total_premium": "2800.00",
      "member_premium": "1000.00",
      "dependent_premium": "1800.00",
      "premiums": {
        "age_banded": {
          "total_member": "1000.00",
          "total_dependent": "1800.00",
          "total": "2800.00"
        },
        "2_tier_composite": {
        "employee_only": "200.00",
        "employee_plus_family": "400.00",
        "total": "2800.00"
        },
        "3_tier_composite": {
          "employee_only": "200.00",
          "employee_plus_one": "250.00",
          "employee_plus_family": "330.00",
          "total": "2800.00"
        },
        "4_tier_composite": {
          "employee_only": "200.00",
          "employee_plus_child": "220.00",
          "employee_plus_spouse": "250.00",
          "employee_plus_family": "330.00",
          "total": "2800.00"
        }
      }
    }
    ...
  ]
}
```


#### Loading Plan Data
Aggregate [Rates](#header-rates) responses *do not* include [Plan](#header-plans) details in order to keep the payload small.  [Plan](#header-plans) data can be retrieved in one of two ways:

Loading the [Plan](#header-plans) from the API:
```
GET /plans/{id}

Response:
{
  "plans": [
    ...
    {
      ...
      "individual_medical_deductible": "$5,000",
      "family_medical_deductible": "$10,000",
      ...
    }
    ...

  ]
}
```
Pulling in [Bulk Plan Data](#bulkplans) and matching up [Plans](#header-plans) by their `id`.

### Retrieving Member-Level Rates
In order to retrieve the exact [Rate](#header-rates) for each `Member` and their `Dependents` for given [Plan](#header-plans), you can load Member-Level `Rates`.

```
GET /rates/{id}/member_rates

Response
{
  "member_rates": [
  ...
  {
    "id": "123abc",
    "member_id": "234def",
    "member_external_id": "externally-supplied-id",
    "member_premium": 500.0,
    "dependent_premium": 600.0,
    "total_premium": 1100.0
  }
  ...
  ]
}

```

Note that *all* `MemberRate`s are for one particular [Plan](#header-plans) - the one referenced by the parent [Rate](#header-rates).

### Business Rules
Vericred works with our Carrier partners to acquire and apply Business Rules that can affect either [Plan](#header-plans) availability or the way in which `Member`s and `Dependent`s are rated.  For example, one Carrier's Business Rules might specify that `Member`s and `Dependent`s who have used tobacco in the past 4 months are considered "tobacco-users", while another's may specify that period to be 1 year.

These rules are applied transparently during the [Quoting](#header-quoting) process and *do not* require any additional action or input on your part.

For a full accounting of Business Rules and a list of Carriers whose Business Rules are applied, please contact support@vericred.com

### Composite Rates
Composite Rates are commonly used in [Major Medical](#header-major-medical), [Dental](#header-dental), and [Vision](#header-vision) [Plans](#header-plans) to simplify operations by charging each family the weighted average of the `Group`'s total premium.  The most common methodology is as follows:

1. Calculate the [Rates](#header-rates) for the entire `Group` using [Sheet Rates](#header-sheet-rates) or [Rate Factors](#header-rate-factors) as appropriate
1. Categorize each Family within the Group.  The categorization differs depending on whether the Composite Rate is 2, 3, or 4-tier
1. Multiply the number of Families in each category by the constant for that category.  These constants are provided to Vericred by the Carrier.  This determines the total number of "Rating Units"
1. Divide the total premium calculated in Step 1 by the total number of Rating Units to get the price per Rating Unit
1. The [Rate](#header-rates) each Family pays is the constant for that Family's category multiplied by the price per Rating Unit.

You can request that a [Quote](#header-quoting) be calculated using Composite Rates when creating it:
```
POST /quotes
{
  ...
  "rating_method": "4_tier_composite"
  ...
}
```
If no Composite Rates methodology is available, the Vericred API will return standard age-banded Rates.

# Network and Provider Data

A `Provider` is an individual or organization in the medical profession.  For example, an individual doctor is a `Provider` as are certain clinics and hospitals.

`Provider`s are related to `Network`s.  A `Network` is a collection of `Provider`s that are under a particular contract with a given `Carrier`.  A given `Carrier` will often have multiple `Network`s.  For example, there may be a large national `Network` as well as several smaller regional `Network`s.

Each `Plan` has a `Network`.  A consumer who visits a `Provider` typically incurs fewer costs when visiting a `Provider` in the `Network` covered by his or her `Plan`.  The `premium` for a `Plan` is often proportional to the size of its `Network`

## Finding Providers
In order to determine if a particular `Plan` covers a given `Provider`, you must first identify the `Provider`.  To do so, use the [Provider Search API endpoint](#providers-providers-post) and specify some search criteria:

```
POST /providers/search
{
  "search_term": "foo",
  "zip_code": "11201"
}
```

The API will return an ordered list of `Provider`s who match the query along with their names, addresses, and other demographic data.  The `id` field returned refers to the `Provider`'s [NPI number](https://www.cms.gov/Regulations-and-Guidance/Administrative-Simplification/NationalProvIdentStand/).  This is the key that is used to identify the `Provider` across different API endpoints.

## Finding Networks

A `Network` is a collection of `Provider`s that are under a particular contract with a given `Carrier`.  A given `Carrier` will often have multiple `Network`s.  For example, there may be a large national `Network` as well as several smaller regional `Network`s.

The API supports searching for `Networks` by Carrier, market and state.  For more details view the [endpoint documentation](#networks-networks-get)

## Matching Providers to Networks
In order to determine if a `Provider` is covered by a user's `Plan`, you will need to map the `Provider` to a `Network`.  There are several methods to do this using the API

### Using Plan Search
You can specify one or more `npi` values in the [`plan` search](#majormedicalplans-majormedicalplans-post).  To do so, include a list of `providers` in the request
```
POST /plans/medical/search
{
  ...
  "providers": [
    { "npi": 1234567890 },
    { "npi": 2345678901 }
  ]
  ...
}

The response will then return a list of `in_network_ids` and `out_of_network_ids` for each `Plan`

```
{
  "plans": [
    ...
    {
      ...
      "id": "12345NY1234567",
      "in_network_ids": [1234567890],
      "out_of_network_ids": [1234567890]
      ...
    },
    {
      ...
      "id": "12345NY2345678",
      "in_network_ids": [1234567890, 1234567890],
      "out_of_network_ids": []
      ...
    }
    ...
  ]
}

Simply reference the `Provider` in question by its `id` for each `Plan` to see if that `Provider` is in-network for the `Plan`.

### Matching by Plan ID
Given a `Provider`'s `id`, you can retrieve all of his or her `hios_ids`

For more details see the [endpoint documentation](#providers-providers-get)

```
GET /providers/1234567890
{
  "provider": {
    ...
    "hios_ids": [
      ...
      "12345NY1234567"
      ...
    ]
    ...
  }
}
```

The returned `hios_ids` can be used to cross-reference a `Plan`

### Matching by Network
Once you have an ID returned from the [`Network` search endpoint](#networks-networks-get), you can cross-reference it with the `network_ids` returned from both the [`Provider` search](#providers-providers-post) and [`Provider` details](#providers-providers-get) endpoints.

This is useful for large group data or when you are not dealing with `Plan`s directly, but rather at the `Network` level.


OpenAPI spec version: 1.0.0

Generated by: https://github.com/swagger-api/swagger-codegen.git

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

=end

require 'date'

module VericredClient

  class RateRequest
    # The age 0 non-tobacco rate for the plan
    attr_accessor :age_0

    # The age 0 tobacco rate for the plan
    attr_accessor :age_0_tobacco

    # The age 1 non-tobacco rate for the plan
    attr_accessor :age_1

    # The age 10 non-tobacco rate for the plan
    attr_accessor :age_10

    # The age 10 tobacco rate for the plan
    attr_accessor :age_10_tobacco

    # The age 11 non-tobacco rate for the plan
    attr_accessor :age_11

    # The age 11 tobacco rate for the plan
    attr_accessor :age_11_tobacco

    # The age 12 non-tobacco rate for the plan
    attr_accessor :age_12

    # The age 12 tobacco rate for the plan
    attr_accessor :age_12_tobacco

    # The age 13 non-tobacco rate for the plan
    attr_accessor :age_13

    # The age 13 tobacco rate for the plan
    attr_accessor :age_13_tobacco

    # The age 14 non-tobacco rate for the plan
    attr_accessor :age_14

    # The age 14 tobacco rate for the plan
    attr_accessor :age_14_tobacco

    # The age 15 non-tobacco rate for the plan
    attr_accessor :age_15

    # The age 15 tobacco rate for the plan
    attr_accessor :age_15_tobacco

    # The age 16 non-tobacco rate for the plan
    attr_accessor :age_16

    # The age 16 tobacco rate for the plan
    attr_accessor :age_16_tobacco

    # The age 17 non-tobacco rate for the plan
    attr_accessor :age_17

    # The age 17 tobacco rate for the plan
    attr_accessor :age_17_tobacco

    # The age 18 non-tobacco rate for the plan
    attr_accessor :age_18

    # The age 18 tobacco rate for the plan
    attr_accessor :age_18_tobacco

    # The age 19 non-tobacco rate for the plan
    attr_accessor :age_19

    # The age 19 tobacco rate for the plan
    attr_accessor :age_19_tobacco

    # The age 1 tobacco rate for the plan
    attr_accessor :age_1_tobacco

    # The age 2 non-tobacco rate for the plan
    attr_accessor :age_2

    # The age 20 non-tobacco rate for the plan
    attr_accessor :age_20

    # The age 20 tobacco rate for the plan
    attr_accessor :age_20_tobacco

    # The age 21 non-tobacco rate for the plan
    attr_accessor :age_21

    # The age 21 tobacco rate for the plan
    attr_accessor :age_21_tobacco

    # The age 22 non-tobacco rate for the plan
    attr_accessor :age_22

    # The age 22 tobacco rate for the plan
    attr_accessor :age_22_tobacco

    # The age 23 non-tobacco rate for the plan
    attr_accessor :age_23

    # The age 23 tobacco rate for the plan
    attr_accessor :age_23_tobacco

    # The age 24 non-tobacco rate for the plan
    attr_accessor :age_24

    # The age 24 tobacco rate for the plan
    attr_accessor :age_24_tobacco

    # The age 25 non-tobacco rate for the plan
    attr_accessor :age_25

    # The age 25 tobacco rate for the plan
    attr_accessor :age_25_tobacco

    # The age 26 non-tobacco rate for the plan
    attr_accessor :age_26

    # The age 26 tobacco rate for the plan
    attr_accessor :age_26_tobacco

    # The age 27 non-tobacco rate for the plan
    attr_accessor :age_27

    # The age 27 tobacco rate for the plan
    attr_accessor :age_27_tobacco

    # The age 28 non-tobacco rate for the plan
    attr_accessor :age_28

    # The age 28 tobacco rate for the plan
    attr_accessor :age_28_tobacco

    # The age 29 non-tobacco rate for the plan
    attr_accessor :age_29

    # The age 29 tobacco rate for the plan
    attr_accessor :age_29_tobacco

    # The age 2 tobacco rate for the plan
    attr_accessor :age_2_tobacco

    # The age 3 non-tobacco rate for the plan
    attr_accessor :age_3

    # The age 30 non-tobacco rate for the plan
    attr_accessor :age_30

    # The age 30 tobacco rate for the plan
    attr_accessor :age_30_tobacco

    # The age 31 non-tobacco rate for the plan
    attr_accessor :age_31

    # The age 31 tobacco rate for the plan
    attr_accessor :age_31_tobacco

    # The age 32 non-tobacco rate for the plan
    attr_accessor :age_32

    # The age 32 tobacco rate for the plan
    attr_accessor :age_32_tobacco

    # The age 33 non-tobacco rate for the plan
    attr_accessor :age_33

    # The age 33 tobacco rate for the plan
    attr_accessor :age_33_tobacco

    # The age 34 non-tobacco rate for the plan
    attr_accessor :age_34

    # The age 34 tobacco rate for the plan
    attr_accessor :age_34_tobacco

    # The age 35 non-tobacco rate for the plan
    attr_accessor :age_35

    # The age 35 tobacco rate for the plan
    attr_accessor :age_35_tobacco

    # The age 36 non-tobacco rate for the plan
    attr_accessor :age_36

    # The age 36 tobacco rate for the plan
    attr_accessor :age_36_tobacco

    # The age 37 non-tobacco rate for the plan
    attr_accessor :age_37

    # The age 37 tobacco rate for the plan
    attr_accessor :age_37_tobacco

    # The age 38 non-tobacco rate for the plan
    attr_accessor :age_38

    # The age 38 tobacco rate for the plan
    attr_accessor :age_38_tobacco

    # The age 39 non-tobacco rate for the plan
    attr_accessor :age_39

    # The age 39 tobacco rate for the plan
    attr_accessor :age_39_tobacco

    # The age 3 tobacco rate for the plan
    attr_accessor :age_3_tobacco

    # The age 4 non-tobacco rate for the plan
    attr_accessor :age_4

    # The age 40 non-tobacco rate for the plan
    attr_accessor :age_40

    # The age 40 tobacco rate for the plan
    attr_accessor :age_40_tobacco

    # The age 41 non-tobacco rate for the plan
    attr_accessor :age_41

    # The age 41 tobacco rate for the plan
    attr_accessor :age_41_tobacco

    # The age 42 non-tobacco rate for the plan
    attr_accessor :age_42

    # The age 42 tobacco rate for the plan
    attr_accessor :age_42_tobacco

    # The age 43 non-tobacco rate for the plan
    attr_accessor :age_43

    # The age 43 tobacco rate for the plan
    attr_accessor :age_43_tobacco

    # The age 44 non-tobacco rate for the plan
    attr_accessor :age_44

    # The age 44 tobacco rate for the plan
    attr_accessor :age_44_tobacco

    # The age 45 non-tobacco rate for the plan
    attr_accessor :age_45

    # The age 45 tobacco rate for the plan
    attr_accessor :age_45_tobacco

    # The age 46 non-tobacco rate for the plan
    attr_accessor :age_46

    # The age 46 tobacco rate for the plan
    attr_accessor :age_46_tobacco

    # The age 47 non-tobacco rate for the plan
    attr_accessor :age_47

    # The age 47 tobacco rate for the plan
    attr_accessor :age_47_tobacco

    # The age 48 non-tobacco rate for the plan
    attr_accessor :age_48

    # The age 48 tobacco rate for the plan
    attr_accessor :age_48_tobacco

    # The age 49 non-tobacco rate for the plan
    attr_accessor :age_49

    # The age 49 tobacco rate for the plan
    attr_accessor :age_49_tobacco

    # The age 4 tobacco rate for the plan
    attr_accessor :age_4_tobacco

    # The age 5 non-tobacco rate for the plan
    attr_accessor :age_5

    # The age 50 non-tobacco rate for the plan
    attr_accessor :age_50

    # The age 50 tobacco rate for the plan
    attr_accessor :age_50_tobacco

    # The age 51 non-tobacco rate for the plan
    attr_accessor :age_51

    # The age 51 tobacco rate for the plan
    attr_accessor :age_51_tobacco

    # The age 52 non-tobacco rate for the plan
    attr_accessor :age_52

    # The age 52 tobacco rate for the plan
    attr_accessor :age_52_tobacco

    # The age 53 non-tobacco rate for the plan
    attr_accessor :age_53

    # The age 53 tobacco rate for the plan
    attr_accessor :age_53_tobacco

    # The age 54 non-tobacco rate for the plan
    attr_accessor :age_54

    # The age 54 tobacco rate for the plan
    attr_accessor :age_54_tobacco

    # The age 55 non-tobacco rate for the plan
    attr_accessor :age_55

    # The age 55 tobacco rate for the plan
    attr_accessor :age_55_tobacco

    # The age 56 non-tobacco rate for the plan
    attr_accessor :age_56

    # The age 56 tobacco rate for the plan
    attr_accessor :age_56_tobacco

    # The age 57 non-tobacco rate for the plan
    attr_accessor :age_57

    # The age 57 tobacco rate for the plan
    attr_accessor :age_57_tobacco

    # The age 58 non-tobacco rate for the plan
    attr_accessor :age_58

    # The age 58 tobacco rate for the plan
    attr_accessor :age_58_tobacco

    # The age 59 non-tobacco rate for the plan
    attr_accessor :age_59

    # The age 59 tobacco rate for the plan
    attr_accessor :age_59_tobacco

    # The age 5 tobacco rate for the plan
    attr_accessor :age_5_tobacco

    # The age 6 non-tobacco rate for the plan
    attr_accessor :age_6

    # The age 60 non-tobacco rate for the plan
    attr_accessor :age_60

    # The age 60 tobacco rate for the plan
    attr_accessor :age_60_tobacco

    # The age 61 non-tobacco rate for the plan
    attr_accessor :age_61

    # The age 61 tobacco rate for the plan
    attr_accessor :age_61_tobacco

    # The age 62 non-tobacco rate for the plan
    attr_accessor :age_62

    # The age 62 tobacco rate for the plan
    attr_accessor :age_62_tobacco

    # The age 63 non-tobacco rate for the plan
    attr_accessor :age_63

    # The age 63 tobacco rate for the plan
    attr_accessor :age_63_tobacco

    # The age 64 non-tobacco rate for the plan
    attr_accessor :age_64

    # The age 64 tobacco rate for the plan
    attr_accessor :age_64_tobacco

    # The age 65 non-tobacco rate for the plan
    attr_accessor :age_65

    # The age 65 tobacco rate for the plan
    attr_accessor :age_65_tobacco

    # The age 6 tobacco rate for the plan
    attr_accessor :age_6_tobacco

    # The age 7 non-tobacco rate for the plan
    attr_accessor :age_7

    # The age 7 tobacco rate for the plan
    attr_accessor :age_7_tobacco

    # The age 8 non-tobacco rate for the plan
    attr_accessor :age_8

    # The age 8 tobacco rate for the plan
    attr_accessor :age_8_tobacco

    # The age 9 non-tobacco rate for the plan
    attr_accessor :age_9

    # The age 9 tobacco rate for the plan
    attr_accessor :age_9_tobacco

    # The tier child_only rate for the plan
    attr_accessor :child_only

    # The effective date of the rate
    attr_accessor :effective_date

    # The expiration date of the rate
    attr_accessor :expiration_date

    # The external id of the plan
    attr_accessor :external_id

    # The tier family rate for the plan
    attr_accessor :family

    # The natural key for the rating area of the  rate
    attr_accessor :rating_area_natural_key

    # The tier single rate for the plan
    attr_accessor :single

    # The tier single_and_children rate for the plan
    attr_accessor :single_and_children

    # The tier single_and_spouse rate for the plan
    attr_accessor :single_and_spouse

    # The document source type of the rate
    attr_accessor :source


    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'age_0' => :'age_0',
        :'age_0_tobacco' => :'age_0_tobacco',
        :'age_1' => :'age_1',
        :'age_10' => :'age_10',
        :'age_10_tobacco' => :'age_10_tobacco',
        :'age_11' => :'age_11',
        :'age_11_tobacco' => :'age_11_tobacco',
        :'age_12' => :'age_12',
        :'age_12_tobacco' => :'age_12_tobacco',
        :'age_13' => :'age_13',
        :'age_13_tobacco' => :'age_13_tobacco',
        :'age_14' => :'age_14',
        :'age_14_tobacco' => :'age_14_tobacco',
        :'age_15' => :'age_15',
        :'age_15_tobacco' => :'age_15_tobacco',
        :'age_16' => :'age_16',
        :'age_16_tobacco' => :'age_16_tobacco',
        :'age_17' => :'age_17',
        :'age_17_tobacco' => :'age_17_tobacco',
        :'age_18' => :'age_18',
        :'age_18_tobacco' => :'age_18_tobacco',
        :'age_19' => :'age_19',
        :'age_19_tobacco' => :'age_19_tobacco',
        :'age_1_tobacco' => :'age_1_tobacco',
        :'age_2' => :'age_2',
        :'age_20' => :'age_20',
        :'age_20_tobacco' => :'age_20_tobacco',
        :'age_21' => :'age_21',
        :'age_21_tobacco' => :'age_21_tobacco',
        :'age_22' => :'age_22',
        :'age_22_tobacco' => :'age_22_tobacco',
        :'age_23' => :'age_23',
        :'age_23_tobacco' => :'age_23_tobacco',
        :'age_24' => :'age_24',
        :'age_24_tobacco' => :'age_24_tobacco',
        :'age_25' => :'age_25',
        :'age_25_tobacco' => :'age_25_tobacco',
        :'age_26' => :'age_26',
        :'age_26_tobacco' => :'age_26_tobacco',
        :'age_27' => :'age_27',
        :'age_27_tobacco' => :'age_27_tobacco',
        :'age_28' => :'age_28',
        :'age_28_tobacco' => :'age_28_tobacco',
        :'age_29' => :'age_29',
        :'age_29_tobacco' => :'age_29_tobacco',
        :'age_2_tobacco' => :'age_2_tobacco',
        :'age_3' => :'age_3',
        :'age_30' => :'age_30',
        :'age_30_tobacco' => :'age_30_tobacco',
        :'age_31' => :'age_31',
        :'age_31_tobacco' => :'age_31_tobacco',
        :'age_32' => :'age_32',
        :'age_32_tobacco' => :'age_32_tobacco',
        :'age_33' => :'age_33',
        :'age_33_tobacco' => :'age_33_tobacco',
        :'age_34' => :'age_34',
        :'age_34_tobacco' => :'age_34_tobacco',
        :'age_35' => :'age_35',
        :'age_35_tobacco' => :'age_35_tobacco',
        :'age_36' => :'age_36',
        :'age_36_tobacco' => :'age_36_tobacco',
        :'age_37' => :'age_37',
        :'age_37_tobacco' => :'age_37_tobacco',
        :'age_38' => :'age_38',
        :'age_38_tobacco' => :'age_38_tobacco',
        :'age_39' => :'age_39',
        :'age_39_tobacco' => :'age_39_tobacco',
        :'age_3_tobacco' => :'age_3_tobacco',
        :'age_4' => :'age_4',
        :'age_40' => :'age_40',
        :'age_40_tobacco' => :'age_40_tobacco',
        :'age_41' => :'age_41',
        :'age_41_tobacco' => :'age_41_tobacco',
        :'age_42' => :'age_42',
        :'age_42_tobacco' => :'age_42_tobacco',
        :'age_43' => :'age_43',
        :'age_43_tobacco' => :'age_43_tobacco',
        :'age_44' => :'age_44',
        :'age_44_tobacco' => :'age_44_tobacco',
        :'age_45' => :'age_45',
        :'age_45_tobacco' => :'age_45_tobacco',
        :'age_46' => :'age_46',
        :'age_46_tobacco' => :'age_46_tobacco',
        :'age_47' => :'age_47',
        :'age_47_tobacco' => :'age_47_tobacco',
        :'age_48' => :'age_48',
        :'age_48_tobacco' => :'age_48_tobacco',
        :'age_49' => :'age_49',
        :'age_49_tobacco' => :'age_49_tobacco',
        :'age_4_tobacco' => :'age_4_tobacco',
        :'age_5' => :'age_5',
        :'age_50' => :'age_50',
        :'age_50_tobacco' => :'age_50_tobacco',
        :'age_51' => :'age_51',
        :'age_51_tobacco' => :'age_51_tobacco',
        :'age_52' => :'age_52',
        :'age_52_tobacco' => :'age_52_tobacco',
        :'age_53' => :'age_53',
        :'age_53_tobacco' => :'age_53_tobacco',
        :'age_54' => :'age_54',
        :'age_54_tobacco' => :'age_54_tobacco',
        :'age_55' => :'age_55',
        :'age_55_tobacco' => :'age_55_tobacco',
        :'age_56' => :'age_56',
        :'age_56_tobacco' => :'age_56_tobacco',
        :'age_57' => :'age_57',
        :'age_57_tobacco' => :'age_57_tobacco',
        :'age_58' => :'age_58',
        :'age_58_tobacco' => :'age_58_tobacco',
        :'age_59' => :'age_59',
        :'age_59_tobacco' => :'age_59_tobacco',
        :'age_5_tobacco' => :'age_5_tobacco',
        :'age_6' => :'age_6',
        :'age_60' => :'age_60',
        :'age_60_tobacco' => :'age_60_tobacco',
        :'age_61' => :'age_61',
        :'age_61_tobacco' => :'age_61_tobacco',
        :'age_62' => :'age_62',
        :'age_62_tobacco' => :'age_62_tobacco',
        :'age_63' => :'age_63',
        :'age_63_tobacco' => :'age_63_tobacco',
        :'age_64' => :'age_64',
        :'age_64_tobacco' => :'age_64_tobacco',
        :'age_65' => :'age_65',
        :'age_65_tobacco' => :'age_65_tobacco',
        :'age_6_tobacco' => :'age_6_tobacco',
        :'age_7' => :'age_7',
        :'age_7_tobacco' => :'age_7_tobacco',
        :'age_8' => :'age_8',
        :'age_8_tobacco' => :'age_8_tobacco',
        :'age_9' => :'age_9',
        :'age_9_tobacco' => :'age_9_tobacco',
        :'child_only' => :'child_only',
        :'effective_date' => :'effective_date',
        :'expiration_date' => :'expiration_date',
        :'external_id' => :'external_id',
        :'family' => :'family',
        :'rating_area_natural_key' => :'rating_area_natural_key',
        :'single' => :'single',
        :'single_and_children' => :'single_and_children',
        :'single_and_spouse' => :'single_and_spouse',
        :'source' => :'source'
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'age_0' => :'Float',
        :'age_0_tobacco' => :'Float',
        :'age_1' => :'Float',
        :'age_10' => :'Float',
        :'age_10_tobacco' => :'Float',
        :'age_11' => :'Float',
        :'age_11_tobacco' => :'Float',
        :'age_12' => :'Float',
        :'age_12_tobacco' => :'Float',
        :'age_13' => :'Float',
        :'age_13_tobacco' => :'Float',
        :'age_14' => :'Float',
        :'age_14_tobacco' => :'Float',
        :'age_15' => :'Float',
        :'age_15_tobacco' => :'Float',
        :'age_16' => :'Float',
        :'age_16_tobacco' => :'Float',
        :'age_17' => :'Float',
        :'age_17_tobacco' => :'Float',
        :'age_18' => :'Float',
        :'age_18_tobacco' => :'Float',
        :'age_19' => :'Float',
        :'age_19_tobacco' => :'Float',
        :'age_1_tobacco' => :'Float',
        :'age_2' => :'Float',
        :'age_20' => :'Float',
        :'age_20_tobacco' => :'Float',
        :'age_21' => :'Float',
        :'age_21_tobacco' => :'Float',
        :'age_22' => :'Float',
        :'age_22_tobacco' => :'Float',
        :'age_23' => :'Float',
        :'age_23_tobacco' => :'Float',
        :'age_24' => :'Float',
        :'age_24_tobacco' => :'Float',
        :'age_25' => :'Float',
        :'age_25_tobacco' => :'Float',
        :'age_26' => :'Float',
        :'age_26_tobacco' => :'Float',
        :'age_27' => :'Float',
        :'age_27_tobacco' => :'Float',
        :'age_28' => :'Float',
        :'age_28_tobacco' => :'Float',
        :'age_29' => :'Float',
        :'age_29_tobacco' => :'Float',
        :'age_2_tobacco' => :'Float',
        :'age_3' => :'Float',
        :'age_30' => :'Float',
        :'age_30_tobacco' => :'Float',
        :'age_31' => :'Float',
        :'age_31_tobacco' => :'Float',
        :'age_32' => :'Float',
        :'age_32_tobacco' => :'Float',
        :'age_33' => :'Float',
        :'age_33_tobacco' => :'Float',
        :'age_34' => :'Float',
        :'age_34_tobacco' => :'Float',
        :'age_35' => :'Float',
        :'age_35_tobacco' => :'Float',
        :'age_36' => :'Float',
        :'age_36_tobacco' => :'Float',
        :'age_37' => :'Float',
        :'age_37_tobacco' => :'Float',
        :'age_38' => :'Float',
        :'age_38_tobacco' => :'Float',
        :'age_39' => :'Float',
        :'age_39_tobacco' => :'Float',
        :'age_3_tobacco' => :'Float',
        :'age_4' => :'Float',
        :'age_40' => :'Float',
        :'age_40_tobacco' => :'Float',
        :'age_41' => :'Float',
        :'age_41_tobacco' => :'Float',
        :'age_42' => :'Float',
        :'age_42_tobacco' => :'Float',
        :'age_43' => :'Float',
        :'age_43_tobacco' => :'Float',
        :'age_44' => :'Float',
        :'age_44_tobacco' => :'Float',
        :'age_45' => :'Float',
        :'age_45_tobacco' => :'Float',
        :'age_46' => :'Float',
        :'age_46_tobacco' => :'Float',
        :'age_47' => :'Float',
        :'age_47_tobacco' => :'Float',
        :'age_48' => :'Float',
        :'age_48_tobacco' => :'Float',
        :'age_49' => :'Float',
        :'age_49_tobacco' => :'Float',
        :'age_4_tobacco' => :'Float',
        :'age_5' => :'Float',
        :'age_50' => :'Float',
        :'age_50_tobacco' => :'Float',
        :'age_51' => :'Float',
        :'age_51_tobacco' => :'Float',
        :'age_52' => :'Float',
        :'age_52_tobacco' => :'Float',
        :'age_53' => :'Float',
        :'age_53_tobacco' => :'Float',
        :'age_54' => :'Float',
        :'age_54_tobacco' => :'Float',
        :'age_55' => :'Float',
        :'age_55_tobacco' => :'Float',
        :'age_56' => :'Float',
        :'age_56_tobacco' => :'Float',
        :'age_57' => :'Float',
        :'age_57_tobacco' => :'Float',
        :'age_58' => :'Float',
        :'age_58_tobacco' => :'Float',
        :'age_59' => :'Float',
        :'age_59_tobacco' => :'Float',
        :'age_5_tobacco' => :'Float',
        :'age_6' => :'Float',
        :'age_60' => :'Float',
        :'age_60_tobacco' => :'Float',
        :'age_61' => :'Float',
        :'age_61_tobacco' => :'Float',
        :'age_62' => :'Float',
        :'age_62_tobacco' => :'Float',
        :'age_63' => :'Float',
        :'age_63_tobacco' => :'Float',
        :'age_64' => :'Float',
        :'age_64_tobacco' => :'Float',
        :'age_65' => :'Float',
        :'age_65_tobacco' => :'Float',
        :'age_6_tobacco' => :'Float',
        :'age_7' => :'Float',
        :'age_7_tobacco' => :'Float',
        :'age_8' => :'Float',
        :'age_8_tobacco' => :'Float',
        :'age_9' => :'Float',
        :'age_9_tobacco' => :'Float',
        :'child_only' => :'Float',
        :'effective_date' => :'String',
        :'expiration_date' => :'String',
        :'external_id' => :'String',
        :'family' => :'Float',
        :'rating_area_natural_key' => :'String',
        :'single' => :'Float',
        :'single_and_children' => :'Float',
        :'single_and_spouse' => :'Float',
        :'source' => :'String'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}){|(k,v), h| h[k.to_sym] = v}

      if attributes.has_key?(:'age_0')
        self.age_0 = attributes[:'age_0']
      end

      if attributes.has_key?(:'age_0_tobacco')
        self.age_0_tobacco = attributes[:'age_0_tobacco']
      end

      if attributes.has_key?(:'age_1')
        self.age_1 = attributes[:'age_1']
      end

      if attributes.has_key?(:'age_10')
        self.age_10 = attributes[:'age_10']
      end

      if attributes.has_key?(:'age_10_tobacco')
        self.age_10_tobacco = attributes[:'age_10_tobacco']
      end

      if attributes.has_key?(:'age_11')
        self.age_11 = attributes[:'age_11']
      end

      if attributes.has_key?(:'age_11_tobacco')
        self.age_11_tobacco = attributes[:'age_11_tobacco']
      end

      if attributes.has_key?(:'age_12')
        self.age_12 = attributes[:'age_12']
      end

      if attributes.has_key?(:'age_12_tobacco')
        self.age_12_tobacco = attributes[:'age_12_tobacco']
      end

      if attributes.has_key?(:'age_13')
        self.age_13 = attributes[:'age_13']
      end

      if attributes.has_key?(:'age_13_tobacco')
        self.age_13_tobacco = attributes[:'age_13_tobacco']
      end

      if attributes.has_key?(:'age_14')
        self.age_14 = attributes[:'age_14']
      end

      if attributes.has_key?(:'age_14_tobacco')
        self.age_14_tobacco = attributes[:'age_14_tobacco']
      end

      if attributes.has_key?(:'age_15')
        self.age_15 = attributes[:'age_15']
      end

      if attributes.has_key?(:'age_15_tobacco')
        self.age_15_tobacco = attributes[:'age_15_tobacco']
      end

      if attributes.has_key?(:'age_16')
        self.age_16 = attributes[:'age_16']
      end

      if attributes.has_key?(:'age_16_tobacco')
        self.age_16_tobacco = attributes[:'age_16_tobacco']
      end

      if attributes.has_key?(:'age_17')
        self.age_17 = attributes[:'age_17']
      end

      if attributes.has_key?(:'age_17_tobacco')
        self.age_17_tobacco = attributes[:'age_17_tobacco']
      end

      if attributes.has_key?(:'age_18')
        self.age_18 = attributes[:'age_18']
      end

      if attributes.has_key?(:'age_18_tobacco')
        self.age_18_tobacco = attributes[:'age_18_tobacco']
      end

      if attributes.has_key?(:'age_19')
        self.age_19 = attributes[:'age_19']
      end

      if attributes.has_key?(:'age_19_tobacco')
        self.age_19_tobacco = attributes[:'age_19_tobacco']
      end

      if attributes.has_key?(:'age_1_tobacco')
        self.age_1_tobacco = attributes[:'age_1_tobacco']
      end

      if attributes.has_key?(:'age_2')
        self.age_2 = attributes[:'age_2']
      end

      if attributes.has_key?(:'age_20')
        self.age_20 = attributes[:'age_20']
      end

      if attributes.has_key?(:'age_20_tobacco')
        self.age_20_tobacco = attributes[:'age_20_tobacco']
      end

      if attributes.has_key?(:'age_21')
        self.age_21 = attributes[:'age_21']
      end

      if attributes.has_key?(:'age_21_tobacco')
        self.age_21_tobacco = attributes[:'age_21_tobacco']
      end

      if attributes.has_key?(:'age_22')
        self.age_22 = attributes[:'age_22']
      end

      if attributes.has_key?(:'age_22_tobacco')
        self.age_22_tobacco = attributes[:'age_22_tobacco']
      end

      if attributes.has_key?(:'age_23')
        self.age_23 = attributes[:'age_23']
      end

      if attributes.has_key?(:'age_23_tobacco')
        self.age_23_tobacco = attributes[:'age_23_tobacco']
      end

      if attributes.has_key?(:'age_24')
        self.age_24 = attributes[:'age_24']
      end

      if attributes.has_key?(:'age_24_tobacco')
        self.age_24_tobacco = attributes[:'age_24_tobacco']
      end

      if attributes.has_key?(:'age_25')
        self.age_25 = attributes[:'age_25']
      end

      if attributes.has_key?(:'age_25_tobacco')
        self.age_25_tobacco = attributes[:'age_25_tobacco']
      end

      if attributes.has_key?(:'age_26')
        self.age_26 = attributes[:'age_26']
      end

      if attributes.has_key?(:'age_26_tobacco')
        self.age_26_tobacco = attributes[:'age_26_tobacco']
      end

      if attributes.has_key?(:'age_27')
        self.age_27 = attributes[:'age_27']
      end

      if attributes.has_key?(:'age_27_tobacco')
        self.age_27_tobacco = attributes[:'age_27_tobacco']
      end

      if attributes.has_key?(:'age_28')
        self.age_28 = attributes[:'age_28']
      end

      if attributes.has_key?(:'age_28_tobacco')
        self.age_28_tobacco = attributes[:'age_28_tobacco']
      end

      if attributes.has_key?(:'age_29')
        self.age_29 = attributes[:'age_29']
      end

      if attributes.has_key?(:'age_29_tobacco')
        self.age_29_tobacco = attributes[:'age_29_tobacco']
      end

      if attributes.has_key?(:'age_2_tobacco')
        self.age_2_tobacco = attributes[:'age_2_tobacco']
      end

      if attributes.has_key?(:'age_3')
        self.age_3 = attributes[:'age_3']
      end

      if attributes.has_key?(:'age_30')
        self.age_30 = attributes[:'age_30']
      end

      if attributes.has_key?(:'age_30_tobacco')
        self.age_30_tobacco = attributes[:'age_30_tobacco']
      end

      if attributes.has_key?(:'age_31')
        self.age_31 = attributes[:'age_31']
      end

      if attributes.has_key?(:'age_31_tobacco')
        self.age_31_tobacco = attributes[:'age_31_tobacco']
      end

      if attributes.has_key?(:'age_32')
        self.age_32 = attributes[:'age_32']
      end

      if attributes.has_key?(:'age_32_tobacco')
        self.age_32_tobacco = attributes[:'age_32_tobacco']
      end

      if attributes.has_key?(:'age_33')
        self.age_33 = attributes[:'age_33']
      end

      if attributes.has_key?(:'age_33_tobacco')
        self.age_33_tobacco = attributes[:'age_33_tobacco']
      end

      if attributes.has_key?(:'age_34')
        self.age_34 = attributes[:'age_34']
      end

      if attributes.has_key?(:'age_34_tobacco')
        self.age_34_tobacco = attributes[:'age_34_tobacco']
      end

      if attributes.has_key?(:'age_35')
        self.age_35 = attributes[:'age_35']
      end

      if attributes.has_key?(:'age_35_tobacco')
        self.age_35_tobacco = attributes[:'age_35_tobacco']
      end

      if attributes.has_key?(:'age_36')
        self.age_36 = attributes[:'age_36']
      end

      if attributes.has_key?(:'age_36_tobacco')
        self.age_36_tobacco = attributes[:'age_36_tobacco']
      end

      if attributes.has_key?(:'age_37')
        self.age_37 = attributes[:'age_37']
      end

      if attributes.has_key?(:'age_37_tobacco')
        self.age_37_tobacco = attributes[:'age_37_tobacco']
      end

      if attributes.has_key?(:'age_38')
        self.age_38 = attributes[:'age_38']
      end

      if attributes.has_key?(:'age_38_tobacco')
        self.age_38_tobacco = attributes[:'age_38_tobacco']
      end

      if attributes.has_key?(:'age_39')
        self.age_39 = attributes[:'age_39']
      end

      if attributes.has_key?(:'age_39_tobacco')
        self.age_39_tobacco = attributes[:'age_39_tobacco']
      end

      if attributes.has_key?(:'age_3_tobacco')
        self.age_3_tobacco = attributes[:'age_3_tobacco']
      end

      if attributes.has_key?(:'age_4')
        self.age_4 = attributes[:'age_4']
      end

      if attributes.has_key?(:'age_40')
        self.age_40 = attributes[:'age_40']
      end

      if attributes.has_key?(:'age_40_tobacco')
        self.age_40_tobacco = attributes[:'age_40_tobacco']
      end

      if attributes.has_key?(:'age_41')
        self.age_41 = attributes[:'age_41']
      end

      if attributes.has_key?(:'age_41_tobacco')
        self.age_41_tobacco = attributes[:'age_41_tobacco']
      end

      if attributes.has_key?(:'age_42')
        self.age_42 = attributes[:'age_42']
      end

      if attributes.has_key?(:'age_42_tobacco')
        self.age_42_tobacco = attributes[:'age_42_tobacco']
      end

      if attributes.has_key?(:'age_43')
        self.age_43 = attributes[:'age_43']
      end

      if attributes.has_key?(:'age_43_tobacco')
        self.age_43_tobacco = attributes[:'age_43_tobacco']
      end

      if attributes.has_key?(:'age_44')
        self.age_44 = attributes[:'age_44']
      end

      if attributes.has_key?(:'age_44_tobacco')
        self.age_44_tobacco = attributes[:'age_44_tobacco']
      end

      if attributes.has_key?(:'age_45')
        self.age_45 = attributes[:'age_45']
      end

      if attributes.has_key?(:'age_45_tobacco')
        self.age_45_tobacco = attributes[:'age_45_tobacco']
      end

      if attributes.has_key?(:'age_46')
        self.age_46 = attributes[:'age_46']
      end

      if attributes.has_key?(:'age_46_tobacco')
        self.age_46_tobacco = attributes[:'age_46_tobacco']
      end

      if attributes.has_key?(:'age_47')
        self.age_47 = attributes[:'age_47']
      end

      if attributes.has_key?(:'age_47_tobacco')
        self.age_47_tobacco = attributes[:'age_47_tobacco']
      end

      if attributes.has_key?(:'age_48')
        self.age_48 = attributes[:'age_48']
      end

      if attributes.has_key?(:'age_48_tobacco')
        self.age_48_tobacco = attributes[:'age_48_tobacco']
      end

      if attributes.has_key?(:'age_49')
        self.age_49 = attributes[:'age_49']
      end

      if attributes.has_key?(:'age_49_tobacco')
        self.age_49_tobacco = attributes[:'age_49_tobacco']
      end

      if attributes.has_key?(:'age_4_tobacco')
        self.age_4_tobacco = attributes[:'age_4_tobacco']
      end

      if attributes.has_key?(:'age_5')
        self.age_5 = attributes[:'age_5']
      end

      if attributes.has_key?(:'age_50')
        self.age_50 = attributes[:'age_50']
      end

      if attributes.has_key?(:'age_50_tobacco')
        self.age_50_tobacco = attributes[:'age_50_tobacco']
      end

      if attributes.has_key?(:'age_51')
        self.age_51 = attributes[:'age_51']
      end

      if attributes.has_key?(:'age_51_tobacco')
        self.age_51_tobacco = attributes[:'age_51_tobacco']
      end

      if attributes.has_key?(:'age_52')
        self.age_52 = attributes[:'age_52']
      end

      if attributes.has_key?(:'age_52_tobacco')
        self.age_52_tobacco = attributes[:'age_52_tobacco']
      end

      if attributes.has_key?(:'age_53')
        self.age_53 = attributes[:'age_53']
      end

      if attributes.has_key?(:'age_53_tobacco')
        self.age_53_tobacco = attributes[:'age_53_tobacco']
      end

      if attributes.has_key?(:'age_54')
        self.age_54 = attributes[:'age_54']
      end

      if attributes.has_key?(:'age_54_tobacco')
        self.age_54_tobacco = attributes[:'age_54_tobacco']
      end

      if attributes.has_key?(:'age_55')
        self.age_55 = attributes[:'age_55']
      end

      if attributes.has_key?(:'age_55_tobacco')
        self.age_55_tobacco = attributes[:'age_55_tobacco']
      end

      if attributes.has_key?(:'age_56')
        self.age_56 = attributes[:'age_56']
      end

      if attributes.has_key?(:'age_56_tobacco')
        self.age_56_tobacco = attributes[:'age_56_tobacco']
      end

      if attributes.has_key?(:'age_57')
        self.age_57 = attributes[:'age_57']
      end

      if attributes.has_key?(:'age_57_tobacco')
        self.age_57_tobacco = attributes[:'age_57_tobacco']
      end

      if attributes.has_key?(:'age_58')
        self.age_58 = attributes[:'age_58']
      end

      if attributes.has_key?(:'age_58_tobacco')
        self.age_58_tobacco = attributes[:'age_58_tobacco']
      end

      if attributes.has_key?(:'age_59')
        self.age_59 = attributes[:'age_59']
      end

      if attributes.has_key?(:'age_59_tobacco')
        self.age_59_tobacco = attributes[:'age_59_tobacco']
      end

      if attributes.has_key?(:'age_5_tobacco')
        self.age_5_tobacco = attributes[:'age_5_tobacco']
      end

      if attributes.has_key?(:'age_6')
        self.age_6 = attributes[:'age_6']
      end

      if attributes.has_key?(:'age_60')
        self.age_60 = attributes[:'age_60']
      end

      if attributes.has_key?(:'age_60_tobacco')
        self.age_60_tobacco = attributes[:'age_60_tobacco']
      end

      if attributes.has_key?(:'age_61')
        self.age_61 = attributes[:'age_61']
      end

      if attributes.has_key?(:'age_61_tobacco')
        self.age_61_tobacco = attributes[:'age_61_tobacco']
      end

      if attributes.has_key?(:'age_62')
        self.age_62 = attributes[:'age_62']
      end

      if attributes.has_key?(:'age_62_tobacco')
        self.age_62_tobacco = attributes[:'age_62_tobacco']
      end

      if attributes.has_key?(:'age_63')
        self.age_63 = attributes[:'age_63']
      end

      if attributes.has_key?(:'age_63_tobacco')
        self.age_63_tobacco = attributes[:'age_63_tobacco']
      end

      if attributes.has_key?(:'age_64')
        self.age_64 = attributes[:'age_64']
      end

      if attributes.has_key?(:'age_64_tobacco')
        self.age_64_tobacco = attributes[:'age_64_tobacco']
      end

      if attributes.has_key?(:'age_65')
        self.age_65 = attributes[:'age_65']
      end

      if attributes.has_key?(:'age_65_tobacco')
        self.age_65_tobacco = attributes[:'age_65_tobacco']
      end

      if attributes.has_key?(:'age_6_tobacco')
        self.age_6_tobacco = attributes[:'age_6_tobacco']
      end

      if attributes.has_key?(:'age_7')
        self.age_7 = attributes[:'age_7']
      end

      if attributes.has_key?(:'age_7_tobacco')
        self.age_7_tobacco = attributes[:'age_7_tobacco']
      end

      if attributes.has_key?(:'age_8')
        self.age_8 = attributes[:'age_8']
      end

      if attributes.has_key?(:'age_8_tobacco')
        self.age_8_tobacco = attributes[:'age_8_tobacco']
      end

      if attributes.has_key?(:'age_9')
        self.age_9 = attributes[:'age_9']
      end

      if attributes.has_key?(:'age_9_tobacco')
        self.age_9_tobacco = attributes[:'age_9_tobacco']
      end

      if attributes.has_key?(:'child_only')
        self.child_only = attributes[:'child_only']
      end

      if attributes.has_key?(:'effective_date')
        self.effective_date = attributes[:'effective_date']
      end

      if attributes.has_key?(:'expiration_date')
        self.expiration_date = attributes[:'expiration_date']
      end

      if attributes.has_key?(:'external_id')
        self.external_id = attributes[:'external_id']
      end

      if attributes.has_key?(:'family')
        self.family = attributes[:'family']
      end

      if attributes.has_key?(:'rating_area_natural_key')
        self.rating_area_natural_key = attributes[:'rating_area_natural_key']
      end

      if attributes.has_key?(:'single')
        self.single = attributes[:'single']
      end

      if attributes.has_key?(:'single_and_children')
        self.single_and_children = attributes[:'single_and_children']
      end

      if attributes.has_key?(:'single_and_spouse')
        self.single_and_spouse = attributes[:'single_and_spouse']
      end

      if attributes.has_key?(:'source')
        self.source = attributes[:'source']
      end

    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properies with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      return invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return true
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          age_0 == o.age_0 &&
          age_0_tobacco == o.age_0_tobacco &&
          age_1 == o.age_1 &&
          age_10 == o.age_10 &&
          age_10_tobacco == o.age_10_tobacco &&
          age_11 == o.age_11 &&
          age_11_tobacco == o.age_11_tobacco &&
          age_12 == o.age_12 &&
          age_12_tobacco == o.age_12_tobacco &&
          age_13 == o.age_13 &&
          age_13_tobacco == o.age_13_tobacco &&
          age_14 == o.age_14 &&
          age_14_tobacco == o.age_14_tobacco &&
          age_15 == o.age_15 &&
          age_15_tobacco == o.age_15_tobacco &&
          age_16 == o.age_16 &&
          age_16_tobacco == o.age_16_tobacco &&
          age_17 == o.age_17 &&
          age_17_tobacco == o.age_17_tobacco &&
          age_18 == o.age_18 &&
          age_18_tobacco == o.age_18_tobacco &&
          age_19 == o.age_19 &&
          age_19_tobacco == o.age_19_tobacco &&
          age_1_tobacco == o.age_1_tobacco &&
          age_2 == o.age_2 &&
          age_20 == o.age_20 &&
          age_20_tobacco == o.age_20_tobacco &&
          age_21 == o.age_21 &&
          age_21_tobacco == o.age_21_tobacco &&
          age_22 == o.age_22 &&
          age_22_tobacco == o.age_22_tobacco &&
          age_23 == o.age_23 &&
          age_23_tobacco == o.age_23_tobacco &&
          age_24 == o.age_24 &&
          age_24_tobacco == o.age_24_tobacco &&
          age_25 == o.age_25 &&
          age_25_tobacco == o.age_25_tobacco &&
          age_26 == o.age_26 &&
          age_26_tobacco == o.age_26_tobacco &&
          age_27 == o.age_27 &&
          age_27_tobacco == o.age_27_tobacco &&
          age_28 == o.age_28 &&
          age_28_tobacco == o.age_28_tobacco &&
          age_29 == o.age_29 &&
          age_29_tobacco == o.age_29_tobacco &&
          age_2_tobacco == o.age_2_tobacco &&
          age_3 == o.age_3 &&
          age_30 == o.age_30 &&
          age_30_tobacco == o.age_30_tobacco &&
          age_31 == o.age_31 &&
          age_31_tobacco == o.age_31_tobacco &&
          age_32 == o.age_32 &&
          age_32_tobacco == o.age_32_tobacco &&
          age_33 == o.age_33 &&
          age_33_tobacco == o.age_33_tobacco &&
          age_34 == o.age_34 &&
          age_34_tobacco == o.age_34_tobacco &&
          age_35 == o.age_35 &&
          age_35_tobacco == o.age_35_tobacco &&
          age_36 == o.age_36 &&
          age_36_tobacco == o.age_36_tobacco &&
          age_37 == o.age_37 &&
          age_37_tobacco == o.age_37_tobacco &&
          age_38 == o.age_38 &&
          age_38_tobacco == o.age_38_tobacco &&
          age_39 == o.age_39 &&
          age_39_tobacco == o.age_39_tobacco &&
          age_3_tobacco == o.age_3_tobacco &&
          age_4 == o.age_4 &&
          age_40 == o.age_40 &&
          age_40_tobacco == o.age_40_tobacco &&
          age_41 == o.age_41 &&
          age_41_tobacco == o.age_41_tobacco &&
          age_42 == o.age_42 &&
          age_42_tobacco == o.age_42_tobacco &&
          age_43 == o.age_43 &&
          age_43_tobacco == o.age_43_tobacco &&
          age_44 == o.age_44 &&
          age_44_tobacco == o.age_44_tobacco &&
          age_45 == o.age_45 &&
          age_45_tobacco == o.age_45_tobacco &&
          age_46 == o.age_46 &&
          age_46_tobacco == o.age_46_tobacco &&
          age_47 == o.age_47 &&
          age_47_tobacco == o.age_47_tobacco &&
          age_48 == o.age_48 &&
          age_48_tobacco == o.age_48_tobacco &&
          age_49 == o.age_49 &&
          age_49_tobacco == o.age_49_tobacco &&
          age_4_tobacco == o.age_4_tobacco &&
          age_5 == o.age_5 &&
          age_50 == o.age_50 &&
          age_50_tobacco == o.age_50_tobacco &&
          age_51 == o.age_51 &&
          age_51_tobacco == o.age_51_tobacco &&
          age_52 == o.age_52 &&
          age_52_tobacco == o.age_52_tobacco &&
          age_53 == o.age_53 &&
          age_53_tobacco == o.age_53_tobacco &&
          age_54 == o.age_54 &&
          age_54_tobacco == o.age_54_tobacco &&
          age_55 == o.age_55 &&
          age_55_tobacco == o.age_55_tobacco &&
          age_56 == o.age_56 &&
          age_56_tobacco == o.age_56_tobacco &&
          age_57 == o.age_57 &&
          age_57_tobacco == o.age_57_tobacco &&
          age_58 == o.age_58 &&
          age_58_tobacco == o.age_58_tobacco &&
          age_59 == o.age_59 &&
          age_59_tobacco == o.age_59_tobacco &&
          age_5_tobacco == o.age_5_tobacco &&
          age_6 == o.age_6 &&
          age_60 == o.age_60 &&
          age_60_tobacco == o.age_60_tobacco &&
          age_61 == o.age_61 &&
          age_61_tobacco == o.age_61_tobacco &&
          age_62 == o.age_62 &&
          age_62_tobacco == o.age_62_tobacco &&
          age_63 == o.age_63 &&
          age_63_tobacco == o.age_63_tobacco &&
          age_64 == o.age_64 &&
          age_64_tobacco == o.age_64_tobacco &&
          age_65 == o.age_65 &&
          age_65_tobacco == o.age_65_tobacco &&
          age_6_tobacco == o.age_6_tobacco &&
          age_7 == o.age_7 &&
          age_7_tobacco == o.age_7_tobacco &&
          age_8 == o.age_8 &&
          age_8_tobacco == o.age_8_tobacco &&
          age_9 == o.age_9 &&
          age_9_tobacco == o.age_9_tobacco &&
          child_only == o.child_only &&
          effective_date == o.effective_date &&
          expiration_date == o.expiration_date &&
          external_id == o.external_id &&
          family == o.family &&
          rating_area_natural_key == o.rating_area_natural_key &&
          single == o.single &&
          single_and_children == o.single_and_children &&
          single_and_spouse == o.single_and_spouse &&
          source == o.source
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [age_0, age_0_tobacco, age_1, age_10, age_10_tobacco, age_11, age_11_tobacco, age_12, age_12_tobacco, age_13, age_13_tobacco, age_14, age_14_tobacco, age_15, age_15_tobacco, age_16, age_16_tobacco, age_17, age_17_tobacco, age_18, age_18_tobacco, age_19, age_19_tobacco, age_1_tobacco, age_2, age_20, age_20_tobacco, age_21, age_21_tobacco, age_22, age_22_tobacco, age_23, age_23_tobacco, age_24, age_24_tobacco, age_25, age_25_tobacco, age_26, age_26_tobacco, age_27, age_27_tobacco, age_28, age_28_tobacco, age_29, age_29_tobacco, age_2_tobacco, age_3, age_30, age_30_tobacco, age_31, age_31_tobacco, age_32, age_32_tobacco, age_33, age_33_tobacco, age_34, age_34_tobacco, age_35, age_35_tobacco, age_36, age_36_tobacco, age_37, age_37_tobacco, age_38, age_38_tobacco, age_39, age_39_tobacco, age_3_tobacco, age_4, age_40, age_40_tobacco, age_41, age_41_tobacco, age_42, age_42_tobacco, age_43, age_43_tobacco, age_44, age_44_tobacco, age_45, age_45_tobacco, age_46, age_46_tobacco, age_47, age_47_tobacco, age_48, age_48_tobacco, age_49, age_49_tobacco, age_4_tobacco, age_5, age_50, age_50_tobacco, age_51, age_51_tobacco, age_52, age_52_tobacco, age_53, age_53_tobacco, age_54, age_54_tobacco, age_55, age_55_tobacco, age_56, age_56_tobacco, age_57, age_57_tobacco, age_58, age_58_tobacco, age_59, age_59_tobacco, age_5_tobacco, age_6, age_60, age_60_tobacco, age_61, age_61_tobacco, age_62, age_62_tobacco, age_63, age_63_tobacco, age_64, age_64_tobacco, age_65, age_65_tobacco, age_6_tobacco, age_7, age_7_tobacco, age_8, age_8_tobacco, age_9, age_9_tobacco, child_only, effective_date, expiration_date, external_id, family, rating_area_natural_key, single, single_and_children, single_and_spouse, source].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = VericredClient.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end

end
